import io
import colorsys

import numpy as np
import PIL

from scipy.ndimage import median_filter


def add_gaussian_noise(img, σ, gray=False):
    Y, X, C = img.shape
    σ = σ / 255
    if σ < 0.:
        gray = True
        σ = -σ
    noise = np.random.normal(0, σ, (Y, X, 1 if gray else 3))
    img = img / 255 + noise
    img[img > 1] = 1
    img[img < 0] = 0
    img = (255 * img).astype(np.uint8)
    return img


def change_hue(img, shift):
    hsv = np.apply_along_axis(lambda x: colorsys.rgb_to_hsv(*x), -1, img)
    hsv[:, :, 0] += shift
    rgb = np.apply_along_axis(lambda x: colorsys.hsv_to_rgb(*x), -1, hsv)
    return rgb.astype(np.uint8)


def gamma_correct(img, γ):
    img = (img / 255)**(1 / γ)
    img = np.clip(img, 0, 1)
    img = (255 * img).astype(np.uint8)
    return img


def jpeg_compress(img, quality):
    pimg = PIL.Image.fromarray(img)
    out = io.BytesIO()
    pimg.save(out, format='JPEG', quality=quality)
    out.seek(0)
    result = np.array(PIL.Image.open(out))
    return result


def apply_median_filter(img):
    footprint = np.array([[0, 1, 0], [1, 1, 1], [0, 1, 0]])[:, :, None]
    return median_filter(img, footprint=footprint)
